@@ -1,10 +1,10 @@
 require "mkmf"
 require "date"
 
-RMAGICK_VERS = "2.11.1"
+RMAGICK_VERS = "2.13.0"
 MIN_RUBY_VERS = "1.8.5"
 MIN_RUBY_VERS_NO = MIN_RUBY_VERS.tr(".","").to_i
-MIN_IM_VERS = "6.3.5"
+MIN_IM_VERS = "6.4.9"
 MIN_IM_VERS_NO = MIN_IM_VERS.tr(".","").to_i
 
 
@@ -73,6 +73,27 @@ def check_multiple_imagemagick_versions()
 end
 
 
+# Ubuntu (maybe other systems) comes with a partial installation of
+# ImageMagick in the prefix /usr (some libraries, no includes, and no
+# binaries). This causes problems when /usr/lib is in the path (e.g., using
+# the default Ruby installation).
+def check_partial_imagemagick_versions()
+   prefix = config_string("prefix")
+   matches = [
+     prefix+"/lib/lib?agick*",
+     prefix+"/include/ImageMagick",
+     prefix+"/bin/Magick-config",
+   ].map do |file_glob|
+     Dir.glob(file_glob)
+   end
+   matches.delete_if { |arr| arr.empty? }
+   if 0 < matches.length and matches.length < 3
+      msg = "\nWarning: Found a partial ImageMagick installation. Your operating system likely has some built-in ImageMagick libraries but not all of ImageMagick. This will most likely cause problems at both compile and runtime.\nFound partial installation at: "+prefix+"\n"
+      Logging::message msg
+      message msg
+   end
+end
+
 
 
 if RUBY_PLATFORM =~ /mswin/
@@ -115,6 +136,7 @@ if RUBY_PLATFORM !~ /mswin|mingw/
   end
 
   check_multiple_imagemagick_versions()
+  check_partial_imagemagick_versions()
 
   # Ensure minimum ImageMagick version
   unless checking_for("ImageMagick version >= #{MIN_IM_VERS}")  do
@@ -194,16 +216,21 @@ have_func("snprintf", headers)
   ["AcquireImage",                   # 6.4.1
    "AffinityImage",                  # 6.4.3-6
    "AffinityImages",                 # 6.4.3-6
+   "AutoGammaImageChannel",          # 6.5.5-1
+   "AutoLevelImageChannel",          # 6.5.5-1
    "BlueShiftImage",                 # 6.5.4-3
+   "ConstituteComponentTerminus",    # 6.5.7-9
    "DeskewImage",                    # 6.4.2-5
    "EncipherImage",                  # 6.3.8-6
    "EqualizeImageChannel",           # 6.3.6-9
    "FloodfillPaintImage",            # 6.3.7
+   "FunctionImageChannel",           # 6.4.8-8
    "GetAuthenticIndexQueue",         # 6.4.5-6
    "GetAuthenticPixels",             # 6.4.5-6
    "GetImageAlphaChannel",           # 6.3.9-2
    "GetVirtualPixels",               # 6.4.5-6
    "LevelImageColors",               # 6.4.2
+   "LevelColorsImageChannel",        # 6.5.6-4
    "LevelizeImageChannel",           # 6.4.2
    "LiquidRescaleImage",             # 6.3.8-2
    "OpaquePaintImageChannel",        # 6.3.7-10
@@ -252,8 +279,10 @@ end
 
 have_struct_member("Image", "type", headers)          # ???
 have_struct_member("DrawInfo", "kerning", headers)    # 6.4.7-8
+have_struct_member("DrawInfo", "interline_spacing", headers)   # 6.5.5-8
 have_struct_member("DrawInfo", "interword_spacing", headers)   # 6.4.8-0
 have_type("DitherMethod", headers)                    # 6.4.2
+have_type("MagickFunction", headers)                  # 6.4.8-8
 have_type("ImageLayerMethod", headers)                # 6.3.6 replaces MagickLayerMethod
 have_type("long double", headers)
 #have_type("unsigned long long", headers)
@@ -314,6 +343,10 @@ have_enum_values("MagickEvaluateOperator", ["PowEvaluateOperator",           # 6
                                            "SineEvaluateOperator",           # 6.4.8-5
                                            "AddModulusEvaluateOperator"],    # 6.4.8-5
                                                                  headers)
+have_enum_values("MagickFunction", ["ArcsinFunction",                        # 6.5.2-8
+                                    "ArctanFunction",                        # 6.5.2-8
+                                    "PolynomialFunction",                    # 6.4.8-8
+                                    "SinusoidFunction"], headers)            # 6.4.8-8
 have_enum_values("ImageLayerMethod", ["FlattenLayer",                           # 6.3.6-2
                                       "MergeLayer",                             # 6.3.6
                                       "MosaicLayer",                            # 6.3.6-2
